/*------------
  region.cpp
  ------------*/

#include "region.h"

#include <stdio.h>
#include <graphics.h>

/*
 class REGION
 */

//-----------------------------------------------------
REGION::REGION(int xx0, int yy0, int xx1, int yy1)
       : x0(xx0), y0(yy0), x1(xx1), y1(yy1)
{ saved = No;
  bitmap = NULL;
  if(x0>x1)
  { int x = x0;
    x0 = x1;
    x1 = x;
  }
  if(y0>y1)
  { int y = y0;
    y0 = y1;
    y1 = y;
  }
  if(x0<0) x0 = 0;
  if(y0<0) y0 = 0;
  if(x1>getmaxx()) x1 = getmaxx();
  if(y1>getmaxy()) y1 = getmaxy();
  unsigned long size =  imagesize( xx0, yy0, xx1, yy1 );
  bitmap = (void far *) new char[size];
}

//-----------------------------------------------------
REGION::~REGION(void)
{ hide();
  delete [] bitmap;
}

//-----------------------------------------------------
int REGION::init(void)
{ return bitmap!=NULL;
}

//-----------------------------------------------------
int REGION::show(void)
{ return show_based() && show_inherited();
}

//-----------------------------------------------------
int REGION::show_based(void)
{ if(!saved && bitmap)
  { getimage(x0,y0,x1,y1,bitmap);
    saved = Yes;
  }
  return Yes;
}

//-----------------------------------------------------
int REGION::hide(void)
{ if(saved && bitmap)
  { putimage(x0,y0,bitmap,COPY_PUT);
    saved = No;
  }
  return Yes;
}

//-----------------------------------------------------
int REGION::work(void)
{ return show();
}

//-----------------------------------------------------
int REGION::move(int xx, int yy)
{ hide();
  int dx, dy, newx, newy;
  dx = xx;
  dy = yy;

  newx = x0 + xx;
  if(newx<0) dx = - x0;
  else
  { newx = x1 + xx;
    if(newx>getmaxx()) dx = getmaxx() - x1;
  }

  newy = y0 + yy;
  if(newy<0) dy = - y0;
  else
  { newy = y1 + yy;
    if(newy>getmaxy()) dy = getmaxy() - y1;
  }

  x0 += dx;
  y0 += dy;
  x1 += dx;
  y1 += dy;
  return show();
}

//-----------------------------------------------------
int REGION::move_to(int xx, int yy)
{ return move(xx-x0, yy-y0);
}
